
/*
 * FSDB_GO/src/fsdb_reader_test.go
 * Copyright (c) 2016-2017 John Heidemann
 * All rights reserved.                                            
 *                                                                
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package fsdb

import (
	"bytes"
//	"fmt"
	"fsdb"
	"io"
	"io/ioutil"
	"strconv"
//	"strings"
	"testing"
)

const (
	sampleFsdb = `#fsdb -F t sid sname
1	dave
2	john
3	sugata
`
)

func TestReader(t *testing.T) {
	ioR := bytes.NewReader([]byte(sampleFsdb))
	in, err := fsdb.NewFsdbReader(ioutil.NopCloser(ioR))
	if err != nil {
		panic("cannot open NewFsdbReader")
	}
	for i := 1; true; i++ {
		row, err := in.ReadRow();
		if err != nil {
			if err != io.EOF {
				panic("error reading rows")
			}
			break
		}
		if got, _ := strconv.Atoi(row[0]); got != i {
			t.Errorf("TestReader: expected %d, got %s", i, row[0])
		}
		// fmt.Println(strings.Join(row, " "))
        }
}

type classTable struct {
	Sid int `col:"sid"`
	Name string `col:"sname"`
}

func TestReaderManualWrapping(t *testing.T) {
	ioR := bytes.NewReader([]byte(sampleFsdb))
	in, err := fsdb.NewFsdbReader(ioutil.NopCloser(ioR))
	if err != nil {
		panic("cannot open NewFsdbReader")
	}

	sidF, _ := in.ColToI("sid")
	nameF, _ := in.ColToI("sname")
	unpackClassTable := func(row []string) classTable {
		var ct classTable
		ct.Sid, _ = strconv.Atoi(row[sidF])
		ct.Name = row[nameF]
		return ct
	}
	for i := 1; true; i++ {
		row, err := in.ReadRow();
		if err != nil {
			if err != io.EOF {
				panic("error reading rows")
			}
			break
		}
		ct := unpackClassTable(row)
		if ct.Sid != i {
			t.Errorf("TestReaderManualWrapping: expected %d, got %d", i, ct.Sid)
		}
		// fmt.Println(strings.Join(row, " "))
        }
}

// TestReaderTwoStepWrapping(t *testing.T): in interal_test

func TestReaderFuncWrapping(t *testing.T) {
	ioR := bytes.NewReader([]byte(sampleFsdb))
	in, err := fsdb.NewFsdbReader(ioutil.NopCloser(ioR))
	if err != nil {
		panic("TestReaderFuncWrapping: cannot open NewFsdbReader")
	}

	var ct classTable
	ioCtRFunc, err := in.MakeReadStructFunc(&ct, "col")
	if err != nil {
		panic("TestReaderFuncWrapping: MakeReadStructState failed")
	}
	for i := 1; true; i++ {
		err = ioCtRFunc(&ct)
		if err != nil {
			if err != io.EOF {
				panic("error reading rows")
			}
			break
		}
		if ct.Sid != i {
			t.Errorf("TestReaderFuncWrapping: expected %d, got %d", i, ct.Sid)
		}
		// fmt.Println(i, ct.Sid, ct.Name)
        }
}

type classTableExtra struct {
	Extra string  // extra field, not to be demarshalled
	Sid int `col:"sid"`
	Name string `col:"sname"`
}

func TestReaderFuncWrappingExtra(t *testing.T) {
	ioR := bytes.NewReader([]byte(sampleFsdb))
	in, err := fsdb.NewFsdbReader(ioutil.NopCloser(ioR))
	if err != nil {
		panic("TestReaderFuncWrapping: cannot open NewFsdbReader")
	}

	var ct classTableExtra
	ioCtRFunc, err := in.MakeReadStructFunc(&ct, "col")
	if err != nil {
		panic("TestReaderFuncWrapping: MakeReadStructState failed")
	}
	for i := 1; true; i++ {
		err = ioCtRFunc(&ct)
		if err != nil {
			if err != io.EOF {
				panic("error reading rows")
			}
			break
		}
		if ct.Sid != i {
			t.Errorf("TestReaderFuncWrapping: expected %d, got %d", i, ct.Sid)
		}
		// fmt.Println(i, ct.Sid, ct.Name)
        }
}
